@@ -5,7 +5,10 @@ from typing import Dict, Any, List, Tuple
 def calculate_roi(detailed_results: pd.DataFrame) -> float:
     """
     Calculate Return on Investment (ROI).
+    Returns 0.0 for empty DataFrames.
     """
+    if len(detailed_results) == 0:
+        return 0.0
     initial_bankroll = detailed_results['bt_starting_bankroll'].iloc[0]
     final_bankroll = detailed_results['bt_ending_bankroll'].iloc[-1]
     return (final_bankroll - initial_bankroll) / initial_bankroll
@@ -56,6 +59,10 @@ def calculate_sharpe_ratio(detailed_results: pd.DataFrame, return_period: int =
     # Annualize the standard deviation by multiplying with sqrt(output_period)
     annualized_std_return = returns.std() * np.sqrt(output_period)
     
+    # Add check for zero standard deviation
+    if annualized_std_return == 0:
+        return 0.0  # or float('inf') if you prefer
+    
     # Calculate and return the Sharpe Ratio
     return annualized_mean_return / annualized_std_return
 
@@ -192,6 +199,10 @@ def calculate_calmar_ratio(detailed_results: pd.DataFrame, return_period: int =
     drawdown = (cumulative_returns - peak) / peak
     max_drawdown = drawdown.min()
     
+    # Add check for zero maximum drawdown
+    if max_drawdown == 0:
+        return 0.0  # or float('inf') if you prefer
+    
     # Annualize the mean return by multiplying with output_period
     annualized_mean_return = returns.mean() * output_period
     
@@ -261,6 +272,190 @@ def calculate_highest_odds(detailed_results: pd.DataFrame) -> Tuple[float, float
     
     return highest_winning_odds, highest_losing_odds
 
+def calculate_avg_roi_per_bet_micro(detailed_results: pd.DataFrame) -> float:
+    """
+    Calculate the micro-average ROI per bet (averaging individual bet ROIs).
+    
+    For each bet, calculates ROI as: (profit / stake) * 100 to get percentage return.
+    Then takes the mean across all bets to get the average ROI per bet.
+    This gives equal weight to each bet's individual ROI.
+    
+    Only considers rows where either:
+    1. A stake was placed (bt_stake > 0), or 
+    2. A bet was made (bt_bet_on != -1)
+    
+    Args:
+        detailed_results (pd.DataFrame): DataFrame containing detailed results of the backtest
+                                       with columns bt_stake, bt_profit, bt_bet_on
+    
+    Returns:
+        float: The micro-average percentage ROI per bet. Returns 0 if no bets were placed.
+        
+    Example:
+        If stakes were $100 each with profits of $20, -$50, $30:
+        ROIs would be: 20%, -50%, 30%
+        Micro-average ROI per bet = 0%
+    """
+    bet_placed = detailed_results[(detailed_results['bt_stake'] > 0) | (detailed_results['bt_bet_on'] != -1)].copy()
+    if bet_placed.empty:
+        return 0
+    # Calculate ROI for each bet: (profit / stake) * 100
+    roi_per_bet = bet_placed['bt_profit'] / bet_placed['bt_stake'] * 100
+    return roi_per_bet.mean() if len(roi_per_bet) > 0 else 0
+
+def calculate_avg_roi_per_bet_macro(detailed_results: pd.DataFrame) -> float:
+    """
+    Calculate the macro-average ROI per bet (total ROI divided by number of bets).
+    
+    Unlike calculate_avg_roi_per_bet_micro which averages individual bet ROIs,
+    this calculates total ROI first and then divides by number of bets.
+    This gives equal weight to the overall return rather than individual bet returns.
+    
+    Args:
+        detailed_results (pd.DataFrame): DataFrame containing detailed results of the backtest
+                                       with columns bt_stake, bt_profit, bt_bet_on
+    
+    Returns:
+        float: Total ROI divided by number of bets. Returns 0 if no bets were placed.
+        
+    Example:
+        If initial bankroll was $1000, final bankroll is $1200 (20% total ROI),
+        and 10 bets were placed, macro-average ROI per bet would be 2% (20% / 10 bets)
+    """
+    bet_placed = detailed_results[(detailed_results['bt_stake'] > 0) | (detailed_results['bt_bet_on'] != -1)]
+    if bet_placed.empty:
+        return 0
+    
+    total_roi = calculate_roi(detailed_results)
+    num_bets = len(bet_placed)
+    
+    return (total_roi / num_bets) * 100 if num_bets > 0 else 0
+
+def calculate_avg_roi_per_year_micro(detailed_results: pd.DataFrame) -> float:
+    """
+    Calculate the micro-average ROI per year by averaging individual yearly ROIs.
+    
+    This function:
+    1. Groups data by year
+    2. Calculates ROI for each year individually
+    3. Takes the average of these yearly ROIs
+    
+    Args:
+        detailed_results (pd.DataFrame): DataFrame containing detailed results of the backtest.
+    
+    Returns:
+        float: The micro-average annual ROI as a percentage (average of individual yearly ROIs).
+        Returns 0.0 for empty DataFrames or when no bets are placed.
+    """
+    if len(detailed_results) == 0:
+        return 0.0
+        
+    # Convert date column to datetime if it's not already
+    dates = pd.to_datetime(detailed_results['bt_date_column'])
+    
+    # Add year column for grouping
+    detailed_results = detailed_results.copy()
+    detailed_results['year'] = dates.dt.year
+    
+    # Group by year and calculate ROI for each year
+    yearly_rois = []
+    for year, group in detailed_results.groupby('year'):
+        initial_bankroll = group['bt_starting_bankroll'].iloc[0]
+        final_bankroll = group['bt_ending_bankroll'].iloc[-1]
+        yearly_roi = ((final_bankroll - initial_bankroll) / initial_bankroll) * 100
+        yearly_rois.append(yearly_roi)
+    
+    # Return average of yearly ROIs
+    return np.mean(yearly_rois) if yearly_rois else 0.0
+
+def calculate_avg_roi_per_year_macro(detailed_results: pd.DataFrame) -> float:
+    """
+    Calculate the macro-average ROI per year by dividing total ROI by number of years.
+    
+    This function:
+    1. Calculates total ROI for the entire period
+    2. Divides by the number of years to get average annual ROI
+    
+    Args:
+        detailed_results (pd.DataFrame): DataFrame containing detailed results of the backtest.
+    
+    Returns:
+        float: The macro-average annual ROI as a percentage (total ROI divided by years).
+        Returns 0.0 for empty DataFrames or when duration is 0.
+    """
+    if len(detailed_results) == 0:
+        return 0.0
+        
+    # Calculate total ROI for the entire period
+    total_roi = calculate_roi(detailed_results)
+    
+    # Convert date column to datetime if it's not already
+    dates = pd.to_datetime(detailed_results['bt_date_column'])
+    
+    # Calculate the number of years (including partial years)
+    years = (dates.max() - dates.min()).days / 365.25
+    
+    # Avoid division by zero
+    if years == 0:
+        return 0.0
+    
+    # Return the annual ROI as a percentage
+    return (total_roi / years) * 100
+
+def calculate_avg_roi_per_year(detailed_results: pd.DataFrame) -> float:
+    """
+    Calculate the average ROI per year by dividing the total ROI by the number of years in the dataset.
+    
+    This function takes the total Return on Investment (ROI) for the entire period and divides it
+    by the number of years in the dataset to get an average annual ROI.
+    Returns 0.0 for empty DataFrames.
+    
+    Args:
+        detailed_results (pd.DataFrame): DataFrame containing detailed results of the backtest.
+    
+    Returns:
+        float: The average annual ROI as a percentage.
+    """
+    if len(detailed_results) == 0:
+        return 0.0
+        
+    # Calculate total ROI for the entire period
+    total_roi = calculate_roi(detailed_results)
+    
+    # Convert date column to datetime if it's not already
+    dates = pd.to_datetime(detailed_results['bt_date_column'])
+    
+    # Calculate the number of years (including partial years)
+    years = (dates.max() - dates.min()).days / 365.25
+    
+    # Avoid division by zero
+    if years == 0:
+        return 0.0
+    
+    # Return the annual ROI as a percentage
+    return (total_roi / years) * 100
+
+def calculate_risk_adjusted_annual_roi(detailed_results: pd.DataFrame) -> float:
+    """
+    Calculate the Risk-Adjusted Annual ROI.
+    This metric divides the average yearly ROI by the maximum drawdown,
+    providing a measure of return per unit of downside risk.
+    A higher value indicates better risk-adjusted annual performance.
+    
+    Returns:
+        float: Risk-Adjusted Annual ROI or 0 if max_drawdown is 0
+    """
+    avg_yearly_roi = calculate_avg_roi_per_year(detailed_results)
+    max_drawdown = calculate_max_drawdown(detailed_results)
+    
+    # Avoid division by zero and handle edge cases
+    if max_drawdown == 0:
+        return 0.0
+    if max_drawdown == -1:  # Complete loss
+        return -avg_yearly_roi  # Return negative value to indicate poor risk-adjusted performance
+    
+    return avg_yearly_roi / abs(max_drawdown)
+
 def calculate_all_metrics(detailed_results: pd.DataFrame) -> Dict[str, Any]:
     """
     Calculate all metrics and return them in a dictionary.
@@ -295,6 +490,11 @@ def calculate_all_metrics(detailed_results: pd.DataFrame) -> Dict[str, Any]:
 
         # Overall Performance
         'ROI [%]': calculate_roi(detailed_results) * 100,
+        'Avg. ROI per Bet [%] (micro)': calculate_avg_roi_per_bet_micro(detailed_results),
+        'Avg. ROI per Bet [%] (macro)': calculate_avg_roi_per_bet_macro(detailed_results),
+        'Avg. ROI per Year [%] (micro)': calculate_avg_roi_per_year_micro(detailed_results),
+        'Avg. ROI per Year [%] (macro)': calculate_avg_roi_per_year_macro(detailed_results),
+        'Risk-Adjusted Annual ROI [-]': calculate_risk_adjusted_annual_roi(detailed_results),
         'Total Profit [$]': calculate_total_profit(detailed_results),
         'Bankroll Final [$]': bankroll_final,
         'Bankroll Peak [$]': bankroll_peak,
